@@ -179,6 +179,39 @@ class Publish(object):
             for source in self._get_source_snapshots(snapshot_remote, fallback_self=True):
                 self.add(source, component)
 
+    def get_packages(self, component=None, components=[], packages=None):
+        """
+        Return package refs for given components
+        """
+        if component:
+            components = [component]
+
+        package_refs = []
+        for snapshot in self.publish_snapshots:
+            if component and snapshot['Component'] not in components:
+                # We don't want packages for this component
+                continue
+
+            component_refs = self.client.do_get('/snapshots/%s/packages' % snapshot['Name'])
+            if packages:
+                # Filter package names
+                for ref in component_refs:
+                    if self.parse_package_ref(ref)[1] in packages:
+                        package_refs.append(ref)
+            else:
+                package_refs.extend(component_refs)
+
+        return package_refs
+
+    def parse_package_ref(self, ref):
+        """
+        Return tuple of architecture, package_name, version, id
+        """
+        if not ref:
+            return None
+        parsed = re.match('(.*)\ (.*)\ (.*)\ (.*)', ref)
+        return parsed.groups()
+
     def add(self, snapshot, component='main'):
         """
         Add snapshot of component to publish
@@ -273,16 +306,19 @@ class Publish(object):
 
         self.client.do_delete('/publish/%s/%s' % (self.prefix, self.distribution))
 
-    def update_publish(self):
+    def update_publish(self, force_overwrite=False):
         lg.info("Updating publish, distribution=%s snapshots=%s" %
                 (self.name, self.publish_snapshots))
 
         self.client.do_put(
             '/publish/%s/%s' % (self.prefix, self.distribution),
-            {'Snapshots': self.publish_snapshots}
+            {
+                'Snapshots': self.publish_snapshots,
+                'ForceOverwrite': force_overwrite,
+            }
         )
 
-    def create_publish(self):
+    def create_publish(self, force_overwrite=False):
         lg.info("Creating new publish, distribution=%s snapshots=%s" %
                 (self.name, self.publish_snapshots))
 
@@ -295,6 +331,7 @@ class Publish(object):
                 "SourceKind": "snapshot",
                 "Distribution": self.distribution,
                 "Sources": self.publish_snapshots,
+                "ForceOverwrite": force_overwrite,
             },
         )
 
@@ -314,7 +351,7 @@ class Publish(object):
                 return publish
         return False
 
-    def do_publish(self, recreate=False):
+    def do_publish(self, recreate=False, force_overwrite=False):
         self.merge_snapshots()
         publish = self.get_publish()
 
@@ -335,6 +372,6 @@ class Publish(object):
                 if recreate:
                     lg.info("Recreating publish %s" % self.name)
                     self.drop_publish()
-                    self.create_publish()
+                    self.create_publish(force_overwrite)
                 else:
-                    self.update_publish()
+                    self.update_publish(force_overwrite)
